<?php
declare(strict_types=1);

namespace Inti\Test\Provider;

use Inti\Provider\Query\QueryTrait;
use RuntimeException;

use function array_combine;
use function array_map;
use function array_search;
use function array_slice;
use function in_array;
use function preg_match;
use function preg_quote;

/**
 * Trait ArrayProvider
 *
 * @package Inti\Test\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
trait ArrayProviderTrait
{
    use QueryTrait;

    public function getCount(): int
    {
        return count($this->getItems());
    }

    public function getItem(): ?array
    {
        return $this->getItems()[0] ?? null;
    }

    public function getItems(): array
    {
        $items = [];
        $fields = array_map('strtolower', $this->fields ?? []);

        $idIndex = array_search('id', $fields);
        $slugIndex = array_search('slug', $fields);
        $titleIndex = array_search('title', $fields);

        foreach ($this->items ?? [] as $item) {
            if (isset($this->ids) && !empty($this->ids)) {
                if (false === $idIndex) {
                    throw new RuntimeException(__METHOD__ . ': Could not detect id field.');
                }

                if (in_array($item[$idIndex], $this->ids)) {
                    $items[] = $item;
                }
            } else if (isset($this->slugs) && !empty($this->slugs)) {
                if (false === $slugIndex) {
                    throw new RuntimeException(__METHOD__ . ': Could not detect slug field.');
                }

                if (in_array($item[$slugIndex], $this->slugs)) {
                    $items[] = $item;
                }
            } else if (isset($this->search) && !empty($this->search)) {
                if (false === $titleIndex) {
                    throw new RuntimeException(__METHOD__ . ': Could not detect title field.');
                }

                if (preg_match('/' . preg_quote($this->search, '/') . '/', $item[$titleIndex])) {
                    $items[] = $item;
                }
            } else {
                $items[] = $item;
            }
        }

        if ($this->offset) {
            $items = array_slice($items, $this->offset);
        }

        if ($this->limit) {
            $items = array_slice($items, 0, $this->limit);
        }

        return array_map(function (array $item) {
            return array_combine($this->fields ?? [], $item);
        }, $items);
    }
}
