<?php
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\App\ConfigProvider;
use Inti\Test\Emitter\TestEmitter;
use Laminas\HttpHandlerRunner\Emitter\EmitterInterface;
use Mezzio\Application;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

/**
 * Class ApplicationTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
abstract class ApplicationTestCase extends ContainerTestCase
{
    use ServerRequestTestTrait;

    private array $providers = [
        ConfigProvider::class,
    ];

    private array $config = [
        Application::class => [
            'inject_default_pipeline' => false,
        ],
        'dependencies' => [
            'invokables' => [
                EmitterInterface::class => TestEmitter::class,
            ],
        ],
    ];

    protected function createApplication(array $config = []): Application
    {
        $container = $this->createContainer($config);
        return $container->get(Application::class);
    }

    protected function runApplication(ServerRequestInterface $request = null, array $config = []): ResponseInterface
    {
        $request = $request ?? $this->createServerRequest();
        $config['dependencies']['services'][ServerRequestInterface::class] = fn() => $request;

        $application = $this->createApplication($config);
        $application->run();

        /** @var TestEmitter $emitter */
        $emitter = $this->_container->get(EmitterInterface::class);
        return $emitter->getReponse();
    }
}
