<?php
/** @noinspection PhpFullyQualifiedNameUsageInspection */
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Inti\Provider\ProviderInterface;
use Inti\Provider\Query\QueryInterface;
use Inti\Provider\Service\ProviderManager;
use RuntimeException;
use function count;

/**
 * Class ProviderTestCase
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
abstract class ProviderTestCase extends ContainerTestCase
{
    const PROVIDER = '';

    /** @var ProviderInterface|QueryInterface */
    protected ProviderInterface $provider;

    private array $providers = [
        \Inti\Provider\ConfigProvider::class,
    ];

    protected function setUp(): void
    {
        parent::setUp();

        if (empty(static::PROVIDER)) {
            throw new RuntimeException('Test class must set the PROVIDER constant.');
        }

        $this->provider = $this->createProvider(static::PROVIDER);
        $this->provider->query();
    }

    protected function createProvider(string $provider): ProviderInterface
    {
        $container = $this->_container ?? $this->createContainer();
        $providerManager = $container->get(ProviderManager::class);
        return $providerManager->get($provider);
    }

    public function testProviderCanGetItems(): array
    {
        $items = $this->provider->getItems();
        $this->assertNotEmpty($items);

        return $items;
    }

    /**
     * @param array $items
     * @depends testProviderCanGetItems
     */
    public function testProviderCanCountItems(array $items)
    {
        $expected = count($items);

        $this->assertEquals($expected, $this->provider->getCount());
    }

    /**
     * @param array $items
     * @depends testProviderCanGetItems
     */
    public function testProviderCanGetSingleItem(array $items)
    {
        $expected = $items[0];

        $this->assertEquals($expected, $this->provider->getItem());
    }
}
