<?php
declare(strict_types=1);

namespace Inti\Test\TestCase;

use Laminas\Diactoros\ServerRequestFactory;
use Psr\Http\Message\ServerRequestInterface;

use function explode;

/**
 * Trait
 *
 * @package Inti\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
trait ServerRequestTestTrait
{
    protected function createServerGetRequest(string $uri = '/'): ServerRequestInterface
    {
        return $this->createServerRequest($uri);
    }

    protected function createServerPostRequest(string $uri, array $data = []): ServerRequestInterface
    {
        return $this->createServerRequest($uri, 'POST', $data);
    }

    protected function createServerRequest(string $uri = '/', string $method = 'GET', array $data = []): ServerRequestInterface
    {
        $queryString = explode('?', $uri, 2)[1] ?? '';

        foreach (explode('&', $queryString) as $param) {
            $param = explode('=', $param, 2);
            $query[$param[0]] = $param[1] ?? '';
        }

        $server = [
            'HTTPS'          => true,
            'QUERY_STRING'   => $queryString,
            'REQUEST_METHOD' => $method,
            'REQUEST_URI'    => $uri,
            'SERVER_NAME'    => $this->hostname ?? 'inti.dev',
        ];

        return ServerRequestFactory::fromGlobals(
            $server,
            $query ?? [],
            $data
        );
    }
}
