<?php
/** @noinspection PhpUndefinedMethodInspection */
/** @noinspection PhpParamsInspection */
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Test\Emitter\TestEmitter;
use Inti\Test\TestCase\ApplicationTestCase;
use Laminas\Diactoros\Response;
use Laminas\HttpHandlerRunner\Emitter\EmitterInterface;
use Mezzio\Application;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;
use Prophecy\Prophecy\ObjectProphecy;
use Psr\Http\Server\MiddlewareInterface;

/**
 * Class ApplicationTest
 *
 * @package IntiTest\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class ApplicationTest extends ApplicationTestCase
{
    use ProphecyTrait;

    public function testIsUsingTheTestEmitter()
    {
        $this->assertContainerHasService(EmitterInterface::class, TestEmitter::class);
    }

    public function testCanCreateTheApplication()
    {
        $app = $this->createApplication();
        $this->assertInstanceOf(Application::class, $app);
    }

    public function testCanGetResponseFromApplication()
    {
        $expected = new Response();

        /** @var MiddlewareInterface|ObjectProphecy $middleware */
        $middleware = $this->prophesize(MiddlewareInterface::class);
        $middleware->process(Argument::any(), Argument::any())
            ->shouldBeCalled()
            ->willReturn($expected);

        $response = $this->runApplication(null, [
            'middleware_pipeline' => [[
                'middleware' => $middleware->reveal(),
                'priority' => 0,
            ]],
        ]);

        $this->assertSame($expected, $response);
    }

    public function testCanProperlyMergeConfiguration()
    {
        $this->createConfig([
            'dependencies' => [
                'services' => [
                    'some_service' => [],
                ],
            ],
        ]);

        $this->assertConfigValueExists('dependencies/services/some_service');
        $this->assertConfigValueExists('dependencies/invokables/' . EmitterInterface::class);
    }
}
