<?php
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Inti\Test\TestCase\ServerRequestTestTrait;
use PHPUnit\Framework\TestCase;

/**
 * Class ServerRequestTest
 *
 * @package IntiTest\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class ServerRequestTest extends TestCase
{
    use ServerRequestTestTrait;

    protected string $hostname = 'example.com';

    public function testRequestsAreSecureByDefault()
    {
        $request = $this->createServerRequest();
        $uri = $request->getUri();

        $this->assertEquals('https', $uri->getScheme());
    }

    public function testCanCreateGetRequest()
    {
        $request = $this->createServerGetRequest('/dir/index.php?param1=value1&param2=value2');

        $this->assertEquals('GET', $request->getMethod());
        $this->assertEquals(['param1' => 'value1', 'param2' => 'value2'], $request->getQueryParams());

        $uri = $request->getUri();

        $this->assertEquals('example.com', $uri->getHost());
        $this->assertEquals('/dir/index.php', $uri->getPath());
        $this->assertEquals('param1=value1&param2=value2', $uri->getQuery());
    }

    public function testCanCreatePostRequest()
    {
        $request = $this->createServerPostRequest('/', ['key' => 'value']);

        $this->assertEquals('POST', $request->getMethod());
        $this->assertEquals(['key' => 'value'], $request->getParsedBody());
    }
}
